@@ -296,7 +296,7 @@ get_ip4_routes (void)
 }
 
 static GVariant *
-get_ip6_routes (void)
+get_ip6_routes (const char *fallback_gateway)
 {
 	gs_unref_ptrarray GPtrArray *routes = NULL;
 	guint i;
@@ -325,7 +325,7 @@ get_ip6_routes (void)
 		nm_sprintf_buf (key_name, "route_ipv6_gateway_%u", i);
 		tmp = getenv (key_name);
 
-		route = nm_ip_route_new (AF_INET6, dst, prefix, tmp, -1, &error);
+		route = nm_ip_route_new (AF_INET6, dst, prefix, tmp ?: fallback_gateway, -1, &error);
 		if (!route) {
 			_LOGW ("Ignoring route#%u: %s", i, error->message);
 			g_error_free (error);
@@ -443,7 +443,7 @@ main (int argc, char *argv[])
 	GError *err = NULL;
 	GPtrArray *dns4_list, *dns6_list;
 	GPtrArray *nbns_list;
-	GPtrArray *dns_domains;
+	GPtrArray *dns_domains, *dns_domain_search;
 	struct in_addr temp_addr;
 	int tapdev = -1;
 	char **iter;
@@ -553,6 +553,10 @@ main (int argc, char *argv[])
 			g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_INT_GATEWAY, val);
 	}
 
+	/* tell Network Manager that openvpn can persist */
+	val = g_variant_new_boolean (TRUE);
+	g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CAN_PERSIST, val);
+
 	/* VPN device */
 	tmp = getenv ("dev");
 	val = str_to_gvariant (tmp, FALSE);
@@ -639,16 +643,6 @@ main (int argc, char *argv[])
 			helper_failed (proxy, "IP6 Address");
 	}
 
-	/* IPv6 remote address */
-	tmp = getenv ("ifconfig_ipv6_remote");
-	if (tmp && strlen (tmp)) {
-		val = addr6_to_gvariant (tmp);
-		if (val)
-			g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_PTP, val);
-		else
-			helper_failed (proxy, "IP6 PTP Address");
-	}
-
 	/* IPv6 netbits */
 	tmp = getenv ("ifconfig_ipv6_netbits");
 	if (tmp && strlen (tmp)) {
@@ -664,7 +658,10 @@ main (int argc, char *argv[])
 		}
 	}
 
-	val = get_ip6_routes ();
+	/* Note: for IPv6 'ifconfig_ipv6_remote' is not used as the peer
+	 * address but as fallback gateway for routes.
+	 */
+	val = get_ip6_routes (getenv ("ifconfig_ipv6_remote"));
 	if (val)
 		g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_ROUTES, val);
 	else if (is_restart) {
@@ -675,6 +672,7 @@ main (int argc, char *argv[])
 
 	/* DNS and WINS servers */
 	dns_domains = g_ptr_array_sized_new (3);
+	dns_domain_search = g_ptr_array_new ();
 	dns4_list = g_ptr_array_new ();
 	dns6_list = g_ptr_array_new ();
 	nbns_list = g_ptr_array_new ();
@@ -702,6 +700,14 @@ main (int argc, char *argv[])
 		else if (   g_str_has_prefix (tmp, "DOMAIN ")
 		         && is_domain_valid (&tmp[NM_STRLEN ("DOMAIN ")]))
 			g_ptr_array_add (dns_domains, &tmp[NM_STRLEN ("DOMAIN ")]);
+		else if (   g_str_has_prefix (tmp, "DOMAIN-SEARCH ")
+		         && is_domain_valid (&tmp[NM_STRLEN ("DOMAIN-SEARCH ")]))
+			g_ptr_array_add (dns_domain_search, &tmp[NM_STRLEN ("DOMAIN-SEARCH ")]);
+	}
+
+	/* if DOMAIN and DOMAIN-SEARCH are intermixed, make the latter follow the former */
+	for (i = 0; i < dns_domain_search->len; i++) {
+		g_ptr_array_add (dns_domains, g_ptr_array_index (dns_domain_search, i));
 	}
 
 	if (dns4_list->len) {
@@ -733,6 +739,7 @@ main (int argc, char *argv[])
 	g_ptr_array_unref (dns4_list);
 	g_ptr_array_unref (dns6_list);
 	g_ptr_array_unref (nbns_list);
+	g_ptr_array_unref (dns_domain_search);
 	g_ptr_array_unref (dns_domains);
 
 	/* Tunnel MTU */
@@ -776,7 +783,7 @@ main (int argc, char *argv[])
 		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_HAS_IP6, val);
 	}
 
-	if (!ip4config && !ip6config)
+	if (!tapdev && !ip4config && !ip6config)
 		helper_failed (proxy, "IPv4 or IPv6 configuration");
 
 	/* Send the config info to nm-openvpn-service */
